import { stat, unlink } from 'fs';
import Mocha from 'mocha';
import { dirname, join, basename } from 'path';

class RollupMochaApiClass {
  constructor(options) {
    const opts = options || {};
    this.dir = opts.dir || opts.file && dirname(opts.file) || "";
    Object.defineProperty(this, "dir", {
      value: this.dir,
      writable: false,
      enumerable: true,
      configurable: false
    });
  }
  instance() {
    return new Mocha();
  }
  noCache(mocha) {
    mocha.suite.on("require", (_, file) => delete require.cache[file]);
    return this;
  }
  getFiles(bundle) {
    const files = [];
    for (const file in bundle) {
      const output = bundle[file];
      if (output.type === "chunk") {
        files.push(join(this.dir, file));
      }
    }
    return files;
  }
  addFiles(files, mocha) {
    for (const file of files) {
      mocha.addFile(file);
    }
    return this;
  }
  validateFiles(files, force = false) {
    let didLogDir = false;
    for (const file of files) {
      if (dirname(file).startsWith(this.dir)) {
        continue;
      }
      if (!force) {
        throw new Error(`File "${basename(file)}" is outside of the output directory "${this.dir}" and cannot be removed.

File: "${file}"
Directory: "${this.dir}"`);
      } else if (!didLogDir) {
        didLogDir = true;
        console.warn("[mocha] Output directory: %o", this.dir);
      }
      console.warn("[mocha] Deleting file outside of output directory: %o", file);
    }
  }
  async removeFiles(files, force = false) {
    this.validateFiles(files, force);
    const promises = files.map(async (file) => {
      const exists = await new Promise((resolve) => {
        stat(file, (error) => resolve(!error));
      });
      if (!exists) {
        return;
      }
      return new Promise((resolve, reject) => {
        unlink(file, (error) => error ? reject(error) : resolve());
      });
    });
    await Promise.all(promises);
  }
  run(runner) {
    return new Promise((resolve, reject) => {
      runner.on("end", () => {
        const { failures } = runner;
        if (failures === 0) {
          return resolve();
        }
        reject(new Error(`Tests failed: ${failures}`));
      });
    });
  }
}

async function run(api, bundle, options) {
  const mocha = options.instance ? await options.instance() : new Mocha();
  if (!options.cache) {
    api.noCache(mocha);
  }
  const allFiles = api.getFiles(bundle);
  const files = options.filterFiles ? options.filterFiles(allFiles) : allFiles;
  try {
    api.addFiles(files, mocha);
    const runner = mocha.run();
    if (options.runner)
      await options.runner(runner, mocha);
    await api.run(runner);
  } finally {
    if (options.clear)
      await api.removeFiles(files);
    if (options.finally)
      await options.finally(mocha);
  }
}

function mocha(pluginOpts = {}) {
  const setup = typeof pluginOpts === "function" ? pluginOpts : (api, bundle) => run(api, bundle, pluginOpts);
  return {
    name: "mocha",
    writeBundle(options, bundle) {
      return setup(new RollupMochaApiClass(options), bundle);
    }
  };
}

export { mocha as default };
